"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.request = exports.waitForDeviceVerification = exports.getDeviceVerificationLink = exports.authorizeWithCode = exports.getAuthorizationCodeLink = void 0;
const axios_1 = __importDefault(require("axios"));
const utils_1 = require("./utils");
function getAuthorizationCodeLink() {
    const { baseUrl, clientId, redirectUri } = this.config;
    return `${baseUrl}/auth/oauth/authorize?response_type=code&client_id=${clientId}&redirect_uri=${redirectUri}`;
}
exports.getAuthorizationCodeLink = getAuthorizationCodeLink;
function handleAuthResponse(client, data) {
    if (!data.access_token) {
        throw data;
    }
    data.expire_ts = data.expires_in + Date.now() / 1000;
    client.config.tokens = data;
    (0, utils_1.storeTokens)(client.config.account, data);
}
function authorizeWithCode(code) {
    return __awaiter(this, void 0, void 0, function* () {
        const { baseUrl, redirectUri, oAuth, account } = this.config;
        try {
            const { data } = yield axios_1.default.post(`${baseUrl}/auth/oauth/token?grant_type=authorization_code&redirect_uri=${redirectUri}&code=${code}`, null, {
                headers: {
                    Authorization: `Basic ${oAuth}`,
                },
            });
            handleAuthResponse(this, data);
            return data;
        }
        catch (err) {
            throw { msg: "authorization_error", err };
        }
    });
}
exports.authorizeWithCode = authorizeWithCode;
function getDeviceVerificationLink() {
    return __awaiter(this, void 0, void 0, function* () {
        const { baseUrl, clientId, oAuth } = this.config;
        const { data: { error, user_code, device_code, expires_in, verification_uri_complete } } = yield axios_1.default.post(`${baseUrl}/auth/oauth/device`, `client_id=${clientId}`, {
            headers: {
                'Authorization': `Basic ${oAuth}`,
                'Content-Type': 'application/x-www-form-urlencoded',
            }
        });
        if (error) {
            throw error;
        }
        this.user_code = user_code;
        this.deviceCode = device_code;
        this.deviceCodeExpiration = new Date(Date.now() + expires_in * 1000);
        return verification_uri_complete;
    });
}
exports.getDeviceVerificationLink = getDeviceVerificationLink;
function waitForDeviceVerification() {
    return __awaiter(this, void 0, void 0, function* () {
        const { baseUrl, oAuth } = this.config;
        while (Date.now() < this.deviceCodeExpiration.getTime()) {
            try {
                const { data } = yield axios_1.default.post(`${baseUrl}/auth/oauth/token?grant_type=urn:ietf:params:oauth:grant-type:device_code&device_code=${this.deviceCode}`, null, {
                    headers: {
                        'Authorization': `Basic ${oAuth}`,
                    },
                });
                handleAuthResponse(this, data);
                return data;
            }
            catch (e) {
                if (e.response.status == 400) {
                    const { error } = e.response.data;
                    if (error === 'authorization_pending') {
                        yield new Promise(resolve => setTimeout(resolve, 5000));
                        continue;
                    }
                    else if (error === 'slow_down') {
                        yield new Promise(resolve => setTimeout(resolve, 10000));
                        continue;
                    }
                    else {
                        throw { msg: "authorization_error", err: error };
                    }
                }
                throw { msg: "authorization_error", err: e };
            }
        }
        throw { msg: "device_code expired", err: null };
    });
}
exports.waitForDeviceVerification = waitForDeviceVerification;
function request(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { apiUrl, tokens } = this.config;
        if (!tokens || !tokens.access_token) {
            return Promise.reject({
                msg: "missing_access_token",
                err: "Access token is missing!",
            });
        }
        if (tokens && tokens.refresh_token && tokens.expire_ts - Date.now() / 1000 < 3600) {
            yield refresh(this.config);
        }
        return new Promise((resolve, reject) => {
            (0, axios_1.default)({
                method: options.method,
                url: `${apiUrl}${options.endpoint}`,
                data: options.data,
                headers: Object.assign({ Authorization: `Bearer ${tokens.access_token}`, Accept: "application/vnd.allegro.public.v1+json" }, options.headers),
            })
                .then((res) => {
                return resolve(res.data);
            })
                .catch((err) => {
                return reject({
                    msg: "request_error",
                    err,
                });
            });
        });
    });
}
exports.request = request;
function refresh(config) {
    const { baseUrl, redirectUri, oAuth, tokens, account } = config;
    if (!tokens || !tokens.refresh_token) {
        return Promise.reject({
            msg: "missing_refresh_token",
            err: "Refresh token is missing",
        });
    }
    return new Promise((resolve, reject) => {
        axios_1.default
            .post(`${baseUrl}/auth/oauth/token?grant_type=refresh_token&refresh_token=${tokens.refresh_token}&redirect_uri=${redirectUri}`, null, {
            headers: { Authorization: `Basic ${oAuth}` },
        })
            .then(({ data }) => {
            if (!data.access_token) {
                return reject({ msg: "authorization_error", err: data });
            }
            data.expire_ts = data.expires_in + Date.now() / 1000;
            config.tokens = data;
            (0, utils_1.storeTokens)(account, data);
            return resolve(data);
        })
            .catch((err) => {
            return reject({
                info: "request_error",
                error: err,
            });
        });
    });
}
//# sourceMappingURL=methods.js.map